from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import Response


def _get_kwargs(
    id: str,
    hash_: str,
    difficulty: str,
    mode: str,
) -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": f"/player/{id}/scorevalue/{hash_}/{difficulty}/{mode}",
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, int]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = cast(int, response.text)
        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = cast(Any, None)
        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, int]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    hash_: str,
    difficulty: str,
    mode: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[Any, int]]:
    """Retrieve player's score for a specific map

     Fetches a score made by a Player with ID for a map specified by Hash and difficulty

    Args:
        id (str):
        hash_ (str):
        difficulty (str):
        mode (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, int]]
    """

    kwargs = _get_kwargs(
        id=id,
        hash_=hash_,
        difficulty=difficulty,
        mode=mode,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    hash_: str,
    difficulty: str,
    mode: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[Any, int]]:
    """Retrieve player's score for a specific map

     Fetches a score made by a Player with ID for a map specified by Hash and difficulty

    Args:
        id (str):
        hash_ (str):
        difficulty (str):
        mode (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, int]
    """

    return sync_detailed(
        id=id,
        hash_=hash_,
        difficulty=difficulty,
        mode=mode,
        client=client,
    ).parsed


async def asyncio_detailed(
    id: str,
    hash_: str,
    difficulty: str,
    mode: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Union[Any, int]]:
    """Retrieve player's score for a specific map

     Fetches a score made by a Player with ID for a map specified by Hash and difficulty

    Args:
        id (str):
        hash_ (str):
        difficulty (str):
        mode (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, int]]
    """

    kwargs = _get_kwargs(
        id=id,
        hash_=hash_,
        difficulty=difficulty,
        mode=mode,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    hash_: str,
    difficulty: str,
    mode: str,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[Union[Any, int]]:
    """Retrieve player's score for a specific map

     Fetches a score made by a Player with ID for a map specified by Hash and difficulty

    Args:
        id (str):
        hash_ (str):
        difficulty (str):
        mode (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, int]
    """

    return (
        await asyncio_detailed(
            id=id,
            hash_=hash_,
            difficulty=difficulty,
            mode=mode,
            client=client,
        )
    ).parsed
